# EPICS IOC Startup Script generated by https://github.com/epics-containers/ibek

cd "/epics/ioc"

epicsEnvSet NAME_AS_ENV_VAR my name is controllerOnePort
epicsEnvSet NAME_AS_ENV_VAR my name is controllerTwoPort

dbLoadDatabase dbd/ioc.dbd
ioc_registerRecordDeviceDriver pdbbase

# Setting up Asyn Port controllerOnePort on 192.168.0.55:2002:
# AsynIPConfigure({{name}}, {{port}}, {{stop}}, {{parity}}, {{bits}}) 
AsynIPConfigure(controllerOnePort, 192.168.0.55:2002, 1, none, 8)
asynSetOption(9600, 0, N, Y)
asynOctetSetInputEos("\n")
asynOctetSetOutputEos("\n")
# Setting up Asyn Port controllerTwoPort on 192.168.0.55:2002:
AsynIPConfigure(controllerTwoPort, 192.168.0.55:2002, 1, none, 8)
asynSetOption(9600, 0, N, Y)
asynOctetSetInputEos("\n")
asynOctetSetOutputEos("\n")

# GAUGE GROUP GAUGE DEFAULTS TEST
# gauge1 P is IBEK-VA-GAUGE-01:
# gauge2 P is IBEK-VA-GAUGE-02:
# gauge3 P is IBEK-VA-GAUGE-01:
# gauge4 P is IBEK-VA-GAUGE-01:
# gauge4 baud is 9600

dbLoadRecords /epics/runtime/ioc.db
iocInit

