# EPICS IOC Startup Script generated by https://github.com/epics-containers/ibek

cd "/epics/ioc"

epicsEnvSet Vec1 192
epicsEnvSet Vec2 193

dbLoadDatabase dbd/ioc.dbd
ioc_registerRecordDeviceDriver pdbbase

# Hy8002 carrier card: slot: 4, intLevel: 2, interrupt: Vec1 (192)
ipacAddHy8002("4, 2, $(Vec1)")
epicsEnvSet IPAC4 = 0
# Hy8002 carrier card: slot: 5, intLevel: 2, interrupt: Vec2 (193)
ipacAddHy8002("5, 2, $(Vec2)")
epicsEnvSet IPAC5 = 1

dbLoadRecords /epics/runtime/ioc.db
iocInit

